==> kmuddy-0.7.1.tar.gz: kmuddy-0.7.1/kmuddy/ctrigger.cpp <==
/***************************************************************************
                          ctrigger.cpp  -  trigger
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne okt 13 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctrigger.h"

#ifdef HAVE_SOUND
#include "csoundplayer.h"
#endif

#include "cansiparser.h"
#include "cconnection.h"
#include "cconnprefs.h"
#include "cconsole.h"
#include "cgroup.h"
#include "ctextchunk.h"
#include "ctextprocessor.h"
#include "ctriggerlist.h"
#include "cvariablelist.h"
#include "kmuddy.h"

#include <kconfig.h>

extern KMuddy *kmuddy;

cTrigger::cTrigger (cSession *s)
{
  sess = s;
  
  text = "";

  //default compare type is substring
  type = substring;

  continueifmatch = false;
  continueifnomatch = true;
  dontsend = false;
  
  colortrigger = false;
  clearColorizations ();

  gagtrigger = false;
  notifytrigger = false;
  prompttrigger = false;
  rewritetrigger = false;
#ifdef HAVE_SOUND
  soundtrigger = false;
#endif

  outputwindowtrigger = false;
  gagoutputwindow = false;
}

cTrigger::~cTrigger ()
{
  //nothing here
}

cSaveableField *cTrigger::newInstance ()
{
  return new cTrigger (sess);
}

bool cTrigger::match (const QString &what)
{
  bool detecting = sess->connection()->getConnPrefs()->triggerList()->
      detectingPrompt ();
  if (detecting != prompttrigger)
    return false;
  return cSaveableField::match (what);
}

//always returns QString::null
QString cTrigger::performAction ()
{
  commandstoexec.clear();

  //colorize the line if needed!
  if (colortrigger && ((!gagtrigger) || prompttrigger))
    recolorize ();

  //rewrite the line if this is a rewrite trigger
  if (rewritetrigger)
    rewrite ();
  
  //prompt-detection triggers are special - no gagging, sounds,
  //notification or cmd sending...
  if (prompttrigger)
  {
    sess->textproc()->setHavePrompt ();
    return QString::null;
  }
  else
  {
    //gag the line if needed
    if (gagtrigger)
      sess->textproc()->gagLine ();

    //notify the user if needed
    if (notifytrigger)
      kmuddy->requestNotify(sess);

    if(outputwindowtrigger)
    {
      sess->textproc()->setOutputWindow(outputwindowname);
      if(gagoutputwindow)
        sess->textproc()->gagLineInPrimary ();
    }

#ifdef HAVE_SOUND
    //attempt to play the sound if needed
    if (soundtrigger && (!(soundFName.isEmpty())))
    {
      cSoundPlayer *player = kmuddy->soundPlayer();
      if (player)
      {
        player->setFileName (soundFName);
        player->setPriority (50);
        player->setRepeatsCount (1);
        player->setVolume (50);
        player->play ();
      }
    }
#endif

    int commands = 0;
    bool haveNonPlain = false;
    QStringList::iterator it;
    for (it = newtext.begin(); it != newtext.end(); ++it)
    {
      QString nt = *it;
      expandPseudoVariables (nt);
      commandstoexec.push_back (nt);
      commands++;
      if (! (nt.isEmpty())) haveNonPlain = true;
    }

    //empty to-sent text
    if ((commands == 1) && (!haveNonPlain))
    {
      //send nothing
      commandstoexec.clear ();
      return QString::null;
    }

    if (dontsend)        
    {
      for (it = commandstoexec.begin(); it != commandstoexec.end(); ++it)
      {
        cTextChunk *chunk = cTextChunk::makeLine (*it, sess->ansiparser()->defaultTextColor(),
            sess->ansiparser()->defaultBkColor(), sess->console());
        sess->console()->addLine (chunk);
      }
      
      commandstoexec.clear ();    //don't send anything!
      return QString::null;
    }
    return QString::null;
  }
}

bool cTrigger::shouldContinue (bool match)
{
  //should we continue?

  //if curent detection mode differs from our prompt-detect setting, then
  //we always continue, as matching was not done
  bool detecting = sess->connection()->getConnPrefs()->triggerList()->
      detectingPrompt ();
  if (detecting != prompttrigger)
    return true;
  
  if ((!nogroup) && (ourgroup != NULL) && (!ourgroup->active()))
    return true;    //if our group is not activated, then we should always
                    //continue (as if we hadn't existed)
  //if the group is active or doesn't exist, use trigger's settings
  return match ? continueifmatch : continueifnomatch;
}

void cTrigger::load (KConfig *config, const QString &group)
{
  config->setGroup (group);
  setText (config->readEntry ("Text", ""));

  //ntxt: for compatibility with KMuddy <= 0.6pre1
  QString ntxt = config->readEntry ("Replacement text", QString::null);

  newtext.clear();
  int repcount = config->readNumEntry ("Replacement count", -1);
  if (repcount == -1)   //use old-style replacement
    newtext.push_back (ntxt);
  else    //new-style replacement
    for (int i = 1; i <= repcount; i++)
    {
      QString repline = config->readEntry ("Replacement line " +
            QString::number (i), "");
      newtext.push_back (repline);
    }

  setType (config->readNumEntry ("Type", substring));
  continueifmatch = config->readBoolEntry ("Continue if match", false);
  continueifnomatch = config->readBoolEntry ("Continue if no match", true);
  dontsend = config->readBoolEntry ("Show dont send", false);
  _group = config->readNumEntry ("Group ID", 0);

  clearColorizations ();
  colortrigger = config->readBoolEntry ("Color trigger", false);
  if (colortrigger)
  {
    int cnum = config->readNumEntry ("Colorizations", 0);
    //ensure that cnum is correct
    if (cnum > MAX_COLORIZATIONS)
      cnum = MAX_COLORIZATIONS;
    if (cnum < 0)
      cnum = 0;
    for (int i = 0; i < cnum; i++)
    {
      QString cname = config->readEntry ("Colorization variable " +
            QString::number(i + 1), "");
      int colors = config->readNumEntry ("Colorization colors " +
            QString::number(i + 1), 255 * 256 + 255);
      QColor fgc = config->readColorEntry ("Colorization fg " +
            QString::number(i + 1), &black);
      QColor bgc = config->readColorEntry ("Colorization bg " +
            QString::number(i + 1), &black);
      addColorization (cname, colors, fgc, bgc);
    }
  }

  rewritetrigger = config->readBoolEntry ("Rewrite trigger", false);
  rewritevar = config->readEntry ("Rewrite variable", QString::null);
  rewritetext = config->readEntry ("Rewrite new text", QString::null);
  
  gagtrigger = config->readBoolEntry ("Gag trigger", false);
  notifytrigger = config->readBoolEntry ("Notify trigger", false);
  prompttrigger = config->readBoolEntry ("Prompt trigger", false);
#ifdef HAVE_SOUND
  soundtrigger = config->readBoolEntry ("Sound trigger", false);
  soundFName = config->readEntry ("Sound file name", QString::null);
#endif

  outputwindowtrigger = config->readBoolEntry ("Send output to separate window", false);
  gagoutputwindow = config->readBoolEntry ("Gag main window in favour of output", false);
  outputwindowname = config->readEntry ("Output window name", QString::null);
}

void cTrigger::save (KConfig *config, const QString &group)
{
  config->setGroup (group);
  config->writeEntry ("Text", text);
  //remove obsolete entry
  config->deleteEntry ("Replacement text");

  int repcount = newtext.count();
  config->writeEntry ("Replacement count", repcount);
  QStringList::iterator it = newtext.begin();
  for (int i = 1; i <= repcount; i++)
  {
    config->writeEntry ("Replacement line " + QString::number (i), *it);
    it++;
  }

  config->writeEntry ("Type", type);
  config->writeEntry ("Continue if match", continueifmatch);
  config->writeEntry ("Continue if no match", continueifnomatch);
  config->writeEntry ("Show dont send", dontsend);
  config->writeEntry ("Group ID", _group);

  config->writeEntry ("Color trigger", colortrigger);
  if (colortrigger)
  {
    for (int i = 0; i < colorizationCount; i++)
    {
      config->writeEntry ("Colorization variable " +
            QString::number(i + 1), colorizationVar[i]);
      config->writeEntry ("Colorization colors " +
            QString::number(i + 1), colorization[i]);
      config->writeEntry ("Colorization fg " +
            QString::number(i + 1), fgcol[i]);
      config->writeEntry ("Colorization bg " +
            QString::number(i + 1), bgcol[i]);
    }
    config->writeEntry ("Colorizations", colorizationCount);
  }

  config->writeEntry ("Rewrite trigger", rewritetrigger);
  config->writeEntry ("Rewrite variable", rewritevar);
  config->writeEntry ("Rewrite new text", rewritetext);

  config->writeEntry ("Gag trigger", gagtrigger);
  config->writeEntry ("Notify trigger", notifytrigger);
  config->writeEntry ("Prompt trigger", prompttrigger);
#ifdef HAVE_SOUND
  config->writeEntry ("Sound trigger", soundtrigger);
  config->writeEntry ("Sound file name", soundFName);
#endif
  config->writeEntry ("Send output to separate window", outputwindowtrigger);
  config->writeEntry ("Gag main window in favour of output", gagoutputwindow);
  config->writeEntry ("Output window name", outputwindowname);
}

void cTrigger::clearColorizations ()
{
  colorizationCount = 0;
  for (int i = 0; i < MAX_COLORIZATIONS; i++)
  {
    colorizationVar[i] = QString::null;
    colorization[i] = 255 * 256 + 255;
    fgcol[i] = black;
    bgcol[i] = black;
  }
}

bool cTrigger::addColorization (QString pseudovar, int newcolors, QColor fgc, QColor bgc)
{
  if (colorizationCount >= MAX_COLORIZATIONS)
    return false;   //NO MORE FREE SPACE!!!
  colorizationVar[colorizationCount] = pseudovar;
  colorization[colorizationCount] = newcolors;
  fgcol[colorizationCount] = fgc;
  bgcol[colorizationCount] = bgc;
  
  colorizationCount++;
  return true;
}

QString cTrigger::getColorizationVariable (int num)
{
  if ((num < 0) || (num >= MAX_COLORIZATIONS))
    return QString::null;
  return colorizationVar[num];
}

int cTrigger::getColorizationColor (int num)
{
  if ((num < 0) || (num >= MAX_COLORIZATIONS))
    return 255 * 256 + 255;
  return colorization[num];
}

QColor cTrigger::getColorizationFg (int num)
{
  if ((num < 0) || (num >= MAX_COLORIZATIONS))
    return black;
  return fgcol[num];
}

QColor cTrigger::getColorizationBg (int num)
{
  if ((num < 0) || (num >= MAX_COLORIZATIONS))
    return black;
  return bgcol[num];
}

void cTrigger::recolorize ()
{
  list<colorChange> colors;
  
  //now go entry by entry :)
  if (colorizationCount > MAX_COLORIZATIONS)  //shouldn't happen, but better be sure...
    colorizationCount = MAX_COLORIZATIONS;
  for (int clr = 0; clr < colorizationCount; clr++)
  {
    QString varname = colorizationVar[clr];
    varname = varname.stripWhiteSpace();
    //varname must start in $
    if (varname[0] != '$')
      continue;
    varname = varname.mid (1);    
    int color = colorization[clr];
    int fgc = color % 256;
    int bgc = color / 256;
    if (fgc == 255) fgc = -1;
    if (bgc == 255) bgc = -1;
    QColor fg, bg;
    if ((fgc >= 0) && (fgc <= 15))
      fg = sess->ansiparser()->color (fgc);
    else if (fgc != -1)
      fg = fgcol[clr];
    if ((bgc >= 0) && (bgc <= 7))
      bg = sess->ansiparser()->color (bgc);
    else if (bgc != -1)
      bg = bgcol[clr];

    computePosition (varname);
    if (fromIndex == -1)  //failed
      continue;
    
    //if we are here, colorization can occur!
    colorChange chg;
    chg.fg = fg;
    chg.bg = bg;
    chg.keepfg = (fgc == -1);
    chg.keepbg = (bgc == -1);
    chg.start = fromIndex;
    chg.len = length;
    colors.push_back (chg);
  }
  
  //okay, recolorize!
  sess->textproc()->recolorize (colors);
}

void cTrigger::rewrite ()
{
  QString varname = rewritevar;
  //trim leading $
  if (varname[0] == '$')
    varname = varname.mid (1);
  computePosition (varname);
  if (fromIndex == -1)
    return;
  
  QString rt = rewritetext;
  //expand pseudo-variables in the replacement text
  expandPseudoVariables (rt);
  //also expand real variables, if any
  rt = sess->connection()->getConnPrefs()->variableList()->expandVariables (rt, true);
  sess->connection()->getConnPrefs()->triggerList()->rewriteText (fromIndex, length, rt);
}

void cTrigger::computePosition (const QString &varname)
{
  //look if we know the variable name...
  bool ok;
  int number = varname.toInt (&ok);
  if (ok)   //it was a number
  {
    if (type == regexp)
    {
      if (number >= (int) backreflist.count())
        fromIndex = -1; //wrong number -> failed
      length = backreflist[number].length();
      fromIndex = backrefpos[number];
    }
    else
      fromIndex = -1;  //failed
  }
  else
  {
    if (varname == "line")
    {
      fromIndex = 0;
      length = lasttext.length();
    }
    else
    if (varname == "prefix")
    {
      fromIndex = 0;
      length = prefix.length();
    }
    else                                      
    if (varname == "suffix")                  
    {                                         
      fromIndex = lasttext.length() - suffix.length();
      length = suffix.length();
    }
    else
    if (varname == "prefixtrim")
    {
      fromIndex = 0;
      length = prefix.stripWhiteSpace().length();
    }
    else
    if (varname == "suffixtrim")
    {
      fromIndex = lasttext.length() - suffix.stripWhiteSpace().length();
      length = suffix.stripWhiteSpace().length();
    }
    else
    if (varname == "matched")
    {
      fromIndex = lastpos;
      length = lastlen;
    }
    else
      fromIndex = -1; //failed
  }
}

==> kmuddy-0.8.tar.gz: kmuddy-0.8/kmuddy/ctrigger.cpp <==
/***************************************************************************
                          ctrigger.cpp  -  trigger
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne okt 13 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctrigger.h"

#ifdef HAVE_SOUND
#include "csoundplayer.h"
#endif

#include "cactionmanager.h"
#include "cansiparser.h"
#include "cgroup.h"
#include "coutput.h"
#include "ctextchunk.h"
#include "ctextprocessor.h"
#include "ctriggerlist.h"
#include "cvariablelist.h"

#include <kconfig.h>

cTrigger::cTrigger (int _sess) : sess(_sess)
{
  text = "";

  //default compare type is substring
  type = substring;

  continueifmatch = false;
  continueifnomatch = true;
  dontsend = false;

  colortrigger = false;
  clearColorizations ();

  gagtrigger = false;
  notifytrigger = false;
  prompttrigger = false;
  rewritetrigger = false;
#ifdef HAVE_SOUND
  soundtrigger = false;
#endif

  outputwindowtrigger = false;
  gagoutputwindow = false;
  
  tl = dynamic_cast<cTriggerList *>(cActionManager::self()->object ("triggerlist", sess));
}

cTrigger::~cTrigger ()
{
  //nothing here
}

cSaveableField *cTrigger::newInstance ()
{
  return new cTrigger (sess);
}

bool cTrigger::match (const QString &what)
{
  bool detecting = tl->detectingPrompt();
  if (detecting != prompttrigger)
    return false;
  return cSaveableField::match (what);
}

//always returns QString::null
QString cTrigger::performAction ()
{
  cTextProcessor *textproc = dynamic_cast<cTextProcessor *>(cActionManager::self()->object ("textproc", sess));
  cOutput *output = dynamic_cast<cOutput *>(cActionManager::self()->object ("output", sess));
  cANSIParser *ansiparser = dynamic_cast<cANSIParser *>(cActionManager::self()->object ("ansiparser", sess));

  commandstoexec.clear();

  //colorize the line if needed!
  if (colortrigger && ((!gagtrigger) || prompttrigger))
    recolorize ();

  //rewrite the line if this is a rewrite trigger
  if (rewritetrigger)
    rewrite ();
  
  //prompt-detection triggers are special - no gagging, sounds,
  //notification or cmd sending...
  if (prompttrigger)
  {
    textproc->setHavePrompt ();
    return QString::null;
  }
  else
  {
    //gag the line if needed
    if (gagtrigger)
      textproc->gagLine ();

    //notify the user if needed
    if (notifytrigger)
      cActionManager::self()->invokeEvent ("notify-request", sess);

    if(outputwindowtrigger)
    {
      textproc->setOutputWindow(outputwindowname);
      if(gagoutputwindow)
        textproc->gagLineInPrimary ();
    }

#ifdef HAVE_SOUND
    //attempt to play the sound if needed
    if (soundtrigger && (!(soundFName.isEmpty())))
    {
      cSoundPlayer *player = dynamic_cast<cSoundPlayer *>(cActionManager::self()->object ("soundplayer", 0));
      if (player)
      {
        player->setFileName (soundFName);
        player->setPriority (50);
        player->setRepeatsCount (1);
        player->setVolume (50);
        player->play ();
      }
    }
#endif

    int commands = 0;
    bool haveNonPlain = false;
    QStringList::iterator it;
    for (it = newtext.begin(); it != newtext.end(); ++it)
    {
      QString nt = *it;
      expandPseudoVariables (nt);
      commandstoexec.push_back (nt);
      commands++;
      if (! (nt.isEmpty())) haveNonPlain = true;
    }

    //empty to-sent text
    if ((commands == 1) && (!haveNonPlain))
    {
      //send nothing
      commandstoexec.clear ();
      return QString::null;
    }

    if (dontsend)        
    {
      for (it = commandstoexec.begin(); it != commandstoexec.end(); ++it)
      {
        cTextChunk *chunk = cTextChunk::makeLine (*it, ansiparser->defaultTextColor(),
            ansiparser->defaultBkColor(), output->console());
        cActionManager::self()->invokeEvent ("display-line", sess, chunk);
        delete chunk;
      }
      
      commandstoexec.clear ();    //don't send anything!
      return QString::null;
    }
    return QString::null;
  }
}

bool cTrigger::shouldContinue (bool match)
{
  //should we continue?

  //if curent detection mode differs from our prompt-detect setting, then
  //we always continue, as matching was not done
  bool detecting = tl->detectingPrompt ();
  if (detecting != prompttrigger)
    return true;
  
  if ((!nogroup) && (ourgroup != NULL) && (!ourgroup->active()))
    return true;    //if our group is not activated, then we should always
                    //continue (as if we hadn't existed)
  //if the group is active or doesn't exist, use trigger's settings
  return match ? continueifmatch : continueifnomatch;
}

void cTrigger::load (KConfig *config, const QString &group)
{
  config->setGroup (group);
  setText (config->readEntry ("Text", ""));

  //ntxt: for compatibility with KMuddy <= 0.6pre1
  QString ntxt = config->readEntry ("Replacement text", QString::null);

  newtext.clear();
  int repcount = config->readNumEntry ("Replacement count", -1);
  if (repcount == -1)   //use old-style replacement
    newtext.push_back (ntxt);
  else    //new-style replacement
    for (int i = 1; i <= repcount; i++)
    {
      QString repline = config->readEntry ("Replacement line " +
            QString::number (i), "");
      newtext.push_back (repline);
    }

  setType (config->readNumEntry ("Type", substring));
  continueifmatch = config->readBoolEntry ("Continue if match", false);
  continueifnomatch = config->readBoolEntry ("Continue if no match", true);
  dontsend = config->readBoolEntry ("Show dont send", false);
  _cs = config->readBoolEntry ("Case sensitive", true);
  globalmatch = config->readBoolEntry ("Global matching", false);
  setCond (config->readEntry ("Condition", QString::null));
  _group = config->readNumEntry ("Group ID", 0);

  clearColorizations ();
  colortrigger = config->readBoolEntry ("Color trigger", false);
  if (colortrigger)
  {
    int cnum = config->readNumEntry ("Colorizations", 0);
    //ensure that cnum is correct
    if (cnum > MAX_COLORIZATIONS)
      cnum = MAX_COLORIZATIONS;
    if (cnum < 0)
      cnum = 0;
    for (int i = 0; i < cnum; i++)
    {
      QString cname = config->readEntry ("Colorization variable " +
            QString::number(i + 1), "");
      int colors = config->readNumEntry ("Colorization colors " +
            QString::number(i + 1), 255 * 256 + 255);
      QColor fgc = config->readColorEntry ("Colorization fg " +
            QString::number(i + 1), &black);
      QColor bgc = config->readColorEntry ("Colorization bg " +
            QString::number(i + 1), &black);
      addColorization (cname, colors, fgc, bgc);
    }
  }

  rewritetrigger = config->readBoolEntry ("Rewrite trigger", false);
  rewritevar = config->readEntry ("Rewrite variable", QString::null);
  rewritetext = config->readEntry ("Rewrite new text", QString::null);
  
  gagtrigger = config->readBoolEntry ("Gag trigger", false);
  notifytrigger = config->readBoolEntry ("Notify trigger", false);
  prompttrigger = config->readBoolEntry ("Prompt trigger", false);
#ifdef HAVE_SOUND
  soundtrigger = config->readBoolEntry ("Sound trigger", false);
  soundFName = config->readEntry ("Sound file name", QString::null);
#endif

  outputwindowtrigger = config->readBoolEntry ("Send output to separate window", false);
  gagoutputwindow = config->readBoolEntry ("Gag main window in favour of output", false);
  outputwindowname = config->readEntry ("Output window name", QString::null);
}

void cTrigger::save (KConfig *config, const QString &group)
{
  config->setGroup (group);
  config->writeEntry ("Text", text);
  //remove obsolete entry
  config->deleteEntry ("Replacement text");

  int repcount = newtext.count();
  config->writeEntry ("Replacement count", repcount);
  QStringList::iterator it = newtext.begin();
  for (int i = 1; i <= repcount; i++)
  {
    config->writeEntry ("Replacement line " + QString::number (i), *it);
    it++;
  }

  config->writeEntry ("Type", type);
  config->writeEntry ("Continue if match", continueifmatch);
  config->writeEntry ("Continue if no match", continueifnomatch);
  config->writeEntry ("Show dont send", dontsend);
  config->writeEntry ("Case sensitive", _cs);
  config->writeEntry ("Global matching", globalmatch);
  config->writeEntry ("Condition", condition);
  config->writeEntry ("Group ID", _group);

  config->writeEntry ("Color trigger", colortrigger);
  if (colortrigger)
  {
    for (int i = 0; i < colorizationCount; i++)
    {
      config->writeEntry ("Colorization variable " +
            QString::number(i + 1), colorizationVar[i]);
      config->writeEntry ("Colorization colors " +
            QString::number(i + 1), colorization[i]);
      config->writeEntry ("Colorization fg " +
            QString::number(i + 1), fgcol[i]);
      config->writeEntry ("Colorization bg " +
            QString::number(i + 1), bgcol[i]);
    }
    config->writeEntry ("Colorizations", colorizationCount);
  }

  config->writeEntry ("Rewrite trigger", rewritetrigger);
  config->writeEntry ("Rewrite variable", rewritevar);
  config->writeEntry ("Rewrite new text", rewritetext);

  config->writeEntry ("Gag trigger", gagtrigger);
  config->writeEntry ("Notify trigger", notifytrigger);
  config->writeEntry ("Prompt trigger", prompttrigger);
#ifdef HAVE_SOUND
  config->writeEntry ("Sound trigger", soundtrigger);
  config->writeEntry ("Sound file name", soundFName);
#endif
  config->writeEntry ("Send output to separate window", outputwindowtrigger);
  config->writeEntry ("Gag main window in favour of output", gagoutputwindow);
  config->writeEntry ("Output window name", outputwindowname);
}

void cTrigger::clearColorizations ()
{
  colorizationCount = 0;
  for (int i = 0; i < MAX_COLORIZATIONS; i++)
  {
    colorizationVar[i] = QString::null;
    colorization[i] = 255 * 256 + 255;
    fgcol[i] = black;
    bgcol[i] = black;
  }
}

bool cTrigger::addColorization (QString pseudovar, int newcolors, QColor fgc, QColor bgc)
{
  if (colorizationCount >= MAX_COLORIZATIONS)
    return false;   //NO MORE FREE SPACE!!!
  colorizationVar[colorizationCount] = pseudovar;
  colorization[colorizationCount] = newcolors;
  fgcol[colorizationCount] = fgc;
  bgcol[colorizationCount] = bgc;
  
  colorizationCount++;
  return true;
}

QString cTrigger::getColorizationVariable (int num)
{
  if ((num < 0) || (num >= MAX_COLORIZATIONS))
    return QString::null;
  return colorizationVar[num];
}

int cTrigger::getColorizationColor (int num)
{
  if ((num < 0) || (num >= MAX_COLORIZATIONS))
    return 255 * 256 + 255;
  return colorization[num];
}

QColor cTrigger::getColorizationFg (int num)
{
  if ((num < 0) || (num >= MAX_COLORIZATIONS))
    return black;
  return fgcol[num];
}

QColor cTrigger::getColorizationBg (int num)
{
  if ((num < 0) || (num >= MAX_COLORIZATIONS))
    return black;
  return bgcol[num];
}

void cTrigger::recolorize ()
{
  cANSIParser *ansiparser = dynamic_cast<cANSIParser *>(cActionManager::self()->object ("ansiparser", sess));
  cTextProcessor *textproc = dynamic_cast<cTextProcessor *>(cActionManager::self()->object ("textproc", sess));
  
  list<colorChange> colors;
  
  //now go entry by entry :)
  if (colorizationCount > MAX_COLORIZATIONS)  //shouldn't happen, but better be sure...
    colorizationCount = MAX_COLORIZATIONS;
  for (int clr = 0; clr < colorizationCount; clr++)
  {
    QString varname = colorizationVar[clr];
    varname = varname.stripWhiteSpace();
    //varname must start in $
    if (varname[0] != '$')
      continue;
    varname = varname.mid (1);    
    int color = colorization[clr];
    int fgc = color % 256;
    int bgc = color / 256;
    if (fgc == 255) fgc = -1;
    if (bgc == 255) bgc = -1;
    QColor fg, bg;
    if ((fgc >= 0) && (fgc <= 15))
      fg = ansiparser->color (fgc);
    else if (fgc != -1)
      fg = fgcol[clr];
    if ((bgc >= 0) && (bgc <= 7))
      bg = ansiparser->color (bgc);
    else if (bgc != -1)
      bg = bgcol[clr];

    computePosition (varname);
    if (fromIndex == -1)  //failed
      continue;
    
    //if we are here, colorization can occur!
    colorChange chg;
    chg.fg = fg;
    chg.bg = bg;
    chg.keepfg = (fgc == -1);
    chg.keepbg = (bgc == -1);
    chg.start = fromIndex;
    chg.len = length;
    colors.push_back (chg);
  }
  
  //okay, recolorize!
  textproc->recolorize (colors);
}

void cTrigger::rewrite ()
{
  QString varname = rewritevar;
  //trim leading $
  if (varname[0] == '$')
    varname = varname.mid (1);
  computePosition (varname);
  if (fromIndex == -1)
    return;
  
  QString rt = rewritetext;
  //expand pseudo-variables in the replacement text
  expandPseudoVariables (rt);
  //also expand real variables, if any
  cVariableList *vl = dynamic_cast<cVariableList *>(cActionManager::self()->object ("variablelist", sess));
  rt = vl->expandVariables (rt, true);
  tl->rewriteText (fromIndex, length, rt);
}

void cTrigger::computePosition (const QString &varname)
{
  //look if we know the variable name...
  bool ok;
  int number = varname.toInt (&ok);
  if (ok)   //it was a number
  {
    if (type == regexp)
    {
      if (number >= (int) backreflist.count())
        fromIndex = -1; //wrong number -> failed
      length = backreflist[number].length();
      fromIndex = backrefpos[number];
    }
    else
      fromIndex = -1;  //failed
  }
  else
  {
    if (varname == "line")
    {
      fromIndex = 0;
      length = lasttext.length();
    }
    else
    if (varname == "prefix")
    {
      fromIndex = 0;
      length = prefix.length();
    }
    else
    if (varname == "suffix")                  
    {                                         
      fromIndex = lasttext.length() - suffix.length();
      length = suffix.length();
    }
    else
    if (varname == "prefixtrim")
    {
      fromIndex = 0;
      length = prefix.stripWhiteSpace().length();
    }
    else
    if (varname == "suffixtrim")
    {
      fromIndex = lasttext.length() - suffix.stripWhiteSpace().length();
      length = suffix.stripWhiteSpace().length();
    }
    else
    if (varname == "matched")
    {
      fromIndex = lastpos;
      length = lastlen;
    }
    else
      fromIndex = -1; //failed
  }
}
